#!/bin/sh
#
# Start the PPTP server daemon
#

PKG_USERCONF_DIR="/usr/syno/etc/packages/VPNPlusServer"
LOCALIP=`grep "^gw-ip-address" ${PKG_USERCONF_DIR}/pptp/accel-pppd.conf | cut -d'=' -f 2`
PKG_TARGET="/var/packages/VPNPlusServer/target"
IPTABLES_MODULE_LIST="/usr/syno/etc.defaults/iptables_modules_list"
SERVICE="pptp"
BIN_IPTABLESTOOL="/usr/syno/bin/iptablestool"
BIN_SYNOMODULETOOL="/usr/syno/bin/synomoduletool"
PID_FILE="/var/run/accel-pppd.pid"
MTU=`/bin/cat ${PKG_USERCONF_DIR}/pptp/accel-pppd.conf | grep mtu | cut -d'=' -f 2`
MSS=$((MTU-44))
GET_KV="/bin/get_key_value"

reverse_modules() {
	local modules=$1
	local ret=""

	for mod in $modules; do
		ret="$mod $ret"
	done

	echo $ret
}

PPTP_Mod=""
if [ -f "${IPTABLES_MODULE_LIST}" ]; then
	. ${IPTABLES_MODULE_LIST}

	for mod in $PPP_MODULES; do
		if [ -e "/lib/modules/$mod" ]; then
			PPTP_Mod="${PPTP_Mod} ${mod}"
		fi
	done

	for mod in $PPTP_MODULES; do
		if [ -e "/lib/modules/$mod" ]; then
			PPTP_Mod="${PPTP_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_CORE; do
		if [ -e "/lib/modules/$mod" ]; then
			PPTP_Mod="${PPTP_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_COMMON; do
		if [ -e "/lib/modules/$mod" ]; then
			PPTP_Mod="${PPTP_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_NAT; do
		if [ -e "/lib/modules/$mod" ]; then
			PPTP_Mod="${PPTP_Mod} ${mod}"
		fi
	done
fi

start() {
	pptpStatus=`/sbin/status pptpServer`

	if [ "${pptpStatus}" != "pptpServer stop/waiting" ]; then
		echo "Already running"
		return
	fi

	echo -n "Starting PPTP server ..."
	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --insmod $SERVICE ${PPTP_Mod}
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		$BIN_IPTABLESTOOL --insmod $SERVICE ${PPTP_Mod}
	fi

	/sbin/start pptpServer
	/sbin/iptables -t mangle -A PRIMITIVE_FORWARD -i ppp2+ -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --set-mss "${MSS}"
	/sbin/iptables -A VPN_SERVER_INPUT -p 47 -j SYNO_INPUT_ACCEPT
	echo 1 > /proc/sys/net/ipv4/ip_forward
}

stop() {
	local modules=`reverse_modules "${PPTP_Mod}"`
	local pptpStatus=`/sbin/status pptpServer`

	if [ "${pptpStatus}" != "pptpServer stop/waiting" ]; then
		# deal with iptable rules
		/sbin/iptables -t mangle -D PRIMITIVE_FORWARD -i ppp2+ -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --set-mss "${MSS}"
		/sbin/iptables -D VPN_SERVER_INPUT -p 47 -j SYNO_INPUT_ACCEPT

		echo -n "Shutting down PPTP server ..."
		/sbin/stop pptpServer
		local time=1

		while true; do
			/bin/sleep 1
			if [ $time -ge 10 ]; then
				break
			fi

			time=$(($time + 1))
			pptpStatus=`/sbin/status pptpServer`
			if [ "${pptpStatus}" = "pptpServer stop/waiting" ]; then
				break
			else
				echo "Wait for stoping pptp server ..."
			fi
		done
	fi

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --rmmod $SERVICE $modules
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		$BIN_IPTABLESTOOL --rmmod $SERVICE $modules
	fi
}

case "$1" in
	start)
		start
	;;
	stop)
		stop
	;;
	restart)
		stop
		start
	;;
	*)
	echo "Usage: $0 {start|stop|restart}"
	exit 1
esac
exit 0
