#!/bin/sh

#####################################################
# This hook is invoked when changing DSM certificate
# Usable environment variable: None
#####################################################

PKG_TARGET="/var/packages/VPNPlusServer/target"
CERT_PY="${PKG_TARGET}/scripts/cert.py"
PKG_USERCONF_DIR="/usr/syno/etc/packages/VPNPlusServer"
CA_BUNDLE="${PKG_USERCONF_DIR}/openvpn/keys/ca_bundle.crt"

CERT_TOOL="/usr/syno/bin/synocerttool"
OPENSSL="/usr/bin/openssl"
BIN_CAT="/bin/cat"
PYTHON="/usr/bin/python2.7"

CreateBundleCA()
{
	local subject=`${OPENSSL} x509 -noout -in $1 -subject | /bin/sed 's/^subject= //g'`
	local issuer=`${OPENSSL} x509 -noout -in $1 -issuer | /bin/sed 's/^issuer= //g'`

	if [ "${subject}" != "${issuer}" ]; then
		for crt in `/usr/bin/find /usr/share/ca-certificates -name "*.crt"`
		do
			subject=`${OPENSSL} x509 -noout -in ${crt} -subject | /bin/sed 's/^subject= //g'`
			if [ "${subject}" = "${issuer}" ]; then
				CreateBundleCA ${crt}
				issuer=`${OPENSSL} x509 -noout -in ${crt} -issuer | /bin/sed 's/^subject= //g'`
				if [ "${subject}" = "${issuer}" ]; then
					break
				fi
			fi
		done
	fi
	cat $1 >> ${CA_BUNDLE}
}

CopyCA() {
	if ${CERT_TOOL} --verify-root-chain; then
		SSL_CA=`${CERT_TOOL} --root-crt`
		NeedBundle="no"

		if [ -f ${PKG_USERCONF_DIR}/no-intercrt ]; then
			rm ${PKG_USERCONF_DIR}/no-intercrt
		fi
	else
		if ${CERT_TOOL} --verify-intercrt-chain; then
			if [ -f ${PKG_USERCONF_DIR}/no-intercrt ]; then
				rm ${PKG_USERCONF_DIR}/no-intercrt
			fi
		else
			touch ${PKG_USERCONF_DIR}/no-intercrt
		fi

		SSL_CA=`${CERT_TOOL} --server-intercrt`
		NeedBundle="yes"
	fi

	SSL_SERVER_CRT=`${CERT_TOOL} --server-crt`
	SSL_SERVER_KEY=`${CERT_TOOL} --server-key`
	cp -f ${SSL_CA} ${PKG_USERCONF_DIR}/openvpn/keys/ca.crt
	cp -f ${SSL_SERVER_CRT} ${PKG_USERCONF_DIR}/openvpn/keys/server.crt
	cp -f ${SSL_SERVER_KEY} ${PKG_USERCONF_DIR}/openvpn/keys/server.key

	cp -f ${SSL_CA} ${PKG_USERCONF_DIR}/certificate/ca.crt
	awk 'BEGIN{i=1}/---BEGIN/{a=1} {if(a) print > "'${PKG_USERCONF_DIR}'/certificate/sslvpn/ca"i".crt";}/---END/{a=0;i=i+1;}' ${SSL_CA}
	cp -f ${SSL_SERVER_CRT} ${PKG_USERCONF_DIR}/certificate/default_ca.crt
	cp -f ${SSL_SERVER_KEY} ${PKG_USERCONF_DIR}/certificate/default_ca.key
	if ${CERT_TOOL} --verify-intercrt-chain; then
		echo -e "\n" >> ${PKG_USERCONF_DIR}/certificate/default_ca.crt;
		cat ${SSL_CA} >> ${PKG_USERCONF_DIR}/certificate/default_ca.crt;
	fi

	# make ca.crt downloadable
	chmod 440 ${PKG_USERCONF_DIR}/certificate/ca.crt

	rm ${CA_BUNDLE}
	if [ "${NeedBundle}" = "yes" ]; then
		# python cert.py {intermediate certificate} {ca path} {output}
		${PYTHON} "${CERT_PY}" "${PKG_USERCONF_DIR}/openvpn/keys/ca.crt" "/usr/share/ca-certificates/" "${CA_BUNDLE}"

		# fallback original design
		if [ $? -eq 1 ]; then
			CreateBundleCA ${PKG_USERCONF_DIR}/openvpn/keys/ca.crt
		fi
	fi
}

case $1 in
	--sdk-mod-ver)
		echo "1.0"
		;;
	--name)
		echo "VPNPlus"
		;;
	--pkg-ver)
		;;
	--vendor)
		;;
	--pre)
		;;
	--post)
		CopyCA
		${PKG_TARGET}/scripts/vpnportal.sh restart
		${PKG_TARGET}/scripts/sslvpn.sh force-restart
		${PKG_TARGET}/scripts/openvpn.sh restart
		${PKG_TARGET}/scripts/remotedesktop.sh restart
		;;
	copyCA)
		CopyCA
		;;
	*)
		echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
		;;
esac
