#!/bin/sh
#
# Control script for IPsec Daemon for L2TP/IPsec and Site-to-Site
#

PKG_USERCONF_DIR="/usr/syno/etc/packages/VPNPlusServer"
PKG_TARGET="/var/packages/VPNPlusServer/target"
IPTABLES_MODULE_LIST="/usr/syno/etc.defaults/iptables_modules_list"
SERVICE="ipsec"
BIN_IPTABLESTOOL="/usr/syno/bin/iptablestool"
BIN_SYNOMODULETOOL="/usr/syno/bin/synomoduletool"
BIN_VPNTOOL="${PKG_TARGET}/tool/synovpnplustool"
SZF_VPNPLUS_CONF="${PKG_USERCONF_DIR}/synovpnplus.conf"
SZF_IPSEC_SITES_CONF="${PKG_USERCONF_DIR}/s2s/sites.conf"
SZF_GET_SECTION_KEY_VALUE="/usr/syno/bin/get_section_key_value"
GET_KV="/bin/get_key_value"
BIN_IPSEC="/usr/sbin/ipsec"

IPSEC_CONF="${PKG_USERCONF_DIR}/ipsec.conf"
IPSEC_SECRETS="${PKG_USERCONF_DIR}/ipsec.secrets"
TUNNEL_PREFIX="tunnel"
IPSEC_TUNNEL_PREFIX="ipsec_${TUNNEL_PREFIX}"
IPSEC_TUNNEL_CONF=".conf"
IPSEC_TUNNEL_SECRETS=".secrets"
L2TP_IPSEC_CONF="${PKG_USERCONF_DIR}/l2tp/ipsec_l2tp.conf"
L2TP_IPSEC_SECRETS="${PKG_USERCONF_DIR}/l2tp/ipsec_l2tp.secrets"

reverse_modules() {
	local modules=$1
	local mod
	local ret=""

	for mod in $modules; do
	    ret="$mod $ret"
	done

	echo $ret
}

IPSec_Mod=""
if [ -f "${IPTABLES_MODULE_LIST}" ]; then
	. ${IPTABLES_MODULE_LIST}

	for mod in $PPP_MODULES; do
		if [ -e "/lib/modules/$mod" ]; then
			IPSec_Mod="${IPSec_Mod} ${mod}"
		fi
	done
	for mod in $L2TP_MODULES; do
		if [ -e "/lib/modules/$mod" ]; then
			IPSec_Mod="${IPSec_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_CORE; do
		if [ -e "/lib/modules/$mod" ]; then
			IPSec_Mod="${IPSec_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_COMMON; do
		if [ -e "/lib/modules/$mod" ]; then
			IPSec_Mod="${IPSec_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_NAT; do
		if [ -e "/lib/modules/$mod" ]; then
			IPSec_Mod="${IPSec_Mod} ${mod}"
		fi
	done
fi

start() {
	ipsecStatus=`/sbin/status ipsec`

	if [ "${ipsecStatus}" != "ipsec stop/waiting" ]; then
		echo "Already running"
		return
	fi

	echo -n "Starting IPsec ..."
	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --insmod $SERVICE ${IPSec_Mod}
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		$BIN_IPTABLESTOOL --insmod $SERVICE ${IPSec_Mod}
	fi

	$BIN_VPNTOOL -s -r

	# copy ipsec.conf and ipsec.secrets to /etc/
	/bin/cp -f ${IPSEC_CONF} /etc/ipsec.conf
	/bin/cp -f ${IPSEC_SECRETS} /etc/ipsec.secrets

	used_id=`${SZF_GET_SECTION_KEY_VALUE} ${SZF_VPNPLUS_CONF} ipsec used_id`
	for i in `seq 0 $((${#used_id}-1))`; do
		if [ "1" = "${used_id:$i:1}" ]; then
			copyTunnelConf $i
		fi
	done

	# deal with iptable rules
	/sbin/iptables -A VPN_SERVER_INPUT -p esp -j SYNO_INPUT_ACCEPT
	/sbin/iptables -A VPN_SERVER_INPUT -p ah -j SYNO_INPUT_ACCEPT

	# clear pluto dir to avoid unexpected shutdown
	/bin/rm -f /var/run/pluto/*

	/sbin/start ipsec

	platform=$(/bin/get_key_value /etc/synoinfo.conf unique | cut -d'_' -f2)
	if [ "$platform" = "dakota" ]; then
		# WAN: core 1, core 3
		for rx_queue in 0 1 2 3
		do
			echo a > /sys/class/net/eth0/queues/rx-$rx_queue/rps_cpus
		done

		# Lan core 2, core 3
		for rx_queue in 0 1 2 3
		do
			echo c > /sys/class/net/eth1/queues/rx-$rx_queue/rps_cpus
		done

		# Reserve core0 for something (crypto ?)
		# for edma_eth_rx0
		rx_irq_num=`grep -m1 edma_eth_rx0 /proc/interrupts | cut -d ':' -f 1 | tail -n1 | tr -d ' '`

		echo e > /proc/irq/$rx_irq_num/smp_affinity
	elif [ "$platform" = "cypress" ]; then
		for eth_interface in 0 1 2 3 4
		do
			for rx_queue in 0 1 2 3
			do
				echo 5 > /sys/class/net/eth$eth_interface/queues/rx-$rx_queue/rps_cpus
			done
		done
		irq_nss_rps=`grep nss_queue0 /proc/interrupts | cut -d ':' -f 1 | tr -d ' '`
		for entry in $irq_nss_rps
		do
			echo 2 > /proc/irq/$entry/smp_affinity
		done

		irq_nss_rps=`grep nss_queue1 /proc/interrupts | cut -d ':' -f 1 | tr -d ' '`
		for entry in $irq_nss_rps
		do
			echo 8 > /proc/irq/$entry/smp_affinity
		done

		irq_nss_rps=`grep nss_queue2 /proc/interrupts | cut -d ':' -f 1 | tr -d ' '`
		for entry in $irq_nss_rps
		do
			echo 2 > /proc/irq/$entry/smp_affinity
		done

		irq_nss_rps=`grep nss_queue3 /proc/interrupts | cut -d ':' -f 1 | tr -d ' '`
		for entry in $irq_nss_rps
		do
			echo 8 > /proc/irq/$entry/smp_affinity
		done
	elif [ "$platform" = "ipq806x" ]; then
		echo 0 >  /sys/class/net/eth0/queues/rx-0/rps_cpus
		echo 0 >  /sys/class/net/eth1/queues/rx-0/rps_cpus
		echo 0 >  /sys/class/net/eth2/queues/rx-0/rps_cpus
	fi
}

stop() {
	local modules=`reverse_modules "${IPSec_Mod}"`

	ipsecStatus=`/sbin/status ipsec`

	if [ "${ipsecStatus}" != "ipsec stop/waiting" ]; then
		# deal with iptable rules
		/sbin/iptables -D VPN_SERVER_INPUT -p esp -j SYNO_INPUT_ACCEPT
		/sbin/iptables -D VPN_SERVER_INPUT -p ah -j SYNO_INPUT_ACCEPT

		/sbin/stop ipsec

		echo -n "Shutting down IPsec ..."
		local time=1

		while true; do
			/bin/sleep 1
			if [ $time -ge 10 ]; then
				break
			fi

			time=$(($time + 1))
			ipsecStatus=`/sbin/status ipsec`

			if [ "${ipsecStatus}" = "ipsec stop/waiting" ]; then
				break
			else
				echo "Wait for stoping IPsec ..."
			fi
		done
	fi

	used_id=`${SZF_GET_SECTION_KEY_VALUE} ${SZF_VPNPLUS_CONF} ipsec used_id`
	for i in `seq 0 $((${#used_id}-1))`; do
		if [ "1" = "${used_id:$i:1}" ]; then
			removeTunnelConf $i
		fi
	done

	# remove /etc/ipsec.conf and /etc/ipsec.secrets to /etc/
	/bin/rm -f  /etc/ipsec.conf
	/bin/rm -f  /etc/ipsec.secrets

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --rmmod $SERVICE $modules
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		$BIN_IPTABLESTOOL --rmmod $SERVICE $modules
	fi

	platform=$(/bin/get_key_value /etc/synoinfo.conf unique | cut -d'_' -f2)
	if [ "$platform" = "dakota" ]; then
		for eth_interface in 0 1
		do
			for rx_queue in 0 1 2 3
			do
				echo 0 > /sys/class/net/eth$eth_interface/queues/rx-$rx_queue/rps_cpus
			done
		done

		# Reserve core0 for something
		# for edma_eth_rx0
		rx_irq_num=`grep -m1 edma_eth_rx0 /proc/interrupts | cut -d ':' -f 1 | tail -n1 | tr -d ' '`

		echo 1 > /proc/irq/$rx_irq_num/smp_affinity
	elif [ "$platform" = "cypress" ]; then
		for eth_interface in 0 1 2 3 4
		do
			for rx_queue in 0 1 2 3
			do
				echo f > /sys/class/net/eth$eth_interface/queues/rx-$rx_queue/rps_cpus
			done
		done

		irq_nss_rps=`grep nss_queue0 /proc/interrupts | cut -d ':' -f 1 | tr -d ' '`
		for entry in $irq_nss_rps
		do
			echo f > /proc/irq/$entry/smp_affinity
		done

		irq_nss_rps=`grep nss_queue1 /proc/interrupts | cut -d ':' -f 1 | tr -d ' '`
		for entry in $irq_nss_rps
		do
			echo 2 > /proc/irq/$entry/smp_affinity
		done

		irq_nss_rps=`grep nss_queue2 /proc/interrupts | cut -d ':' -f 1 | tr -d ' '`
		for entry in $irq_nss_rps
		do
			echo 4 > /proc/irq/$entry/smp_affinity
		done

		irq_nss_rps=`grep nss_queue3 /proc/interrupts | cut -d ':' -f 1 | tr -d ' '`
		for entry in $irq_nss_rps
		do
			echo 8 > /proc/irq/$entry/smp_affinity
		done

	elif [ "$platform" = "ipq806x" ]; then
		echo 3 >  /sys/class/net/eth0/queues/rx-0/rps_cpus
		echo 3 >  /sys/class/net/eth1/queues/rx-0/rps_cpus
		echo 3 >  /sys/class/net/eth2/queues/rx-0/rps_cpus
	fi
}

addtunnel ()
{
	copyTunnelConf "$1"
	TUNNEL_NAME="${TUNNEL_PREFIX}$1"

	local ipsecStatus
	ipsecStatus=$(/sbin/status ipsec)

	if [ "${ipsecStatus}" = "ipsec stop/waiting" ]; then
		start
	else
		${BIN_IPSEC} addconn --checkconfig
		${BIN_IPSEC} auto --add "${TUNNEL_NAME}"
		${BIN_IPSEC} whack --unlisten
		${BIN_IPSEC} whack --listen
		${BIN_IPSEC} auto --start "${TUNNEL_NAME}" &
	fi
}

deltunnel()
{
	TUNNEL_NAME="${TUNNEL_PREFIX}$1"

	${BIN_IPSEC} auto --delete ${TUNNEL_NAME}

	removeTunnelConf $1

	${BIN_IPSEC} addconn --checkconfig
	${BIN_IPSEC} auto --rereadsecrets
}

copyTunnelConf()
{
	TUNNEL_ID=$1
	ENABLE=`jq -r --arg key $1 '.[] | select(.site_id == $key) | .enable' ${SZF_IPSEC_SITES_CONF}`
	REMOTE_LANS=`jq -r --arg key $1 '.[] | select(.site_id == $key) | .remote_lan | .[]' ${SZF_IPSEC_SITES_CONF}`
	TUNNEL_CONF="${PKG_USERCONF_DIR}/s2s/${IPSEC_TUNNEL_PREFIX}${TUNNEL_ID}${IPSEC_TUNNEL_CONF}"
	TUNNEL_SECRETS="${PKG_USERCONF_DIR}/s2s/${IPSEC_TUNNEL_PREFIX}${TUNNEL_ID}${IPSEC_TUNNEL_SECRETS}"

	if [ "${ENABLE}" = "false" ]; then
		return
	fi

	/bin/cp -f ${TUNNEL_CONF} /etc/ipsec.d/
	/bin/chmod 400 ${TUNNEL_SECRETS}
	/bin/cp -f ${TUNNEL_SECRETS} /etc/ipsec.d/
	/bin/rm -f ${TUNNEL_SECRETS}

	# deal with iptable and route rules
	for REMOTE_LAN in ${REMOTE_LANS}; do
		iptables -t nat -I VPNPLUS_SNAT_POSTROUTING -d ${REMOTE_LAN} -j MARK --set-xmark 0x10000000/0x10000000
		iptables -t filter -A VPNPLUS_S2S_ALLOW_FORWARD -s ${REMOTE_LAN} -j SYNO_FORWARD_ACCEPT
	done
}

removeTunnelConf()
{
	TUNNEL_ID=$1
	ENABLE=`jq -r --arg key $1 '.[] | select(.site_id == $key) | .enable' ${SZF_IPSEC_SITES_CONF}`
	REMOTE_LANS=`jq -r --arg key $1 '.[] | select(.site_id == $key) | .remote_lan | .[]' ${SZF_IPSEC_SITES_CONF}`
	TUNNEL_NAME="${TUNNEL_PREFIX}${TUNNEL_ID}"
	TUNNEL_CONF="${IPSEC_TUNNEL_PREFIX}${TUNNEL_ID}${IPSEC_TUNNEL_CONF}"
	TUNNEL_SECRETS="${IPSEC_TUNNEL_PREFIX}${TUNNEL_ID}${IPSEC_TUNNEL_SECRETS}"

	if [ "${ENABLE}" = "false" ]; then
		return
	fi

	/bin/rm -f /etc/ipsec.d/${TUNNEL_CONF}
	/bin/rm -f /etc/ipsec.d/${TUNNEL_SECRETS}

	# deal with iptable and route rules
	for REMOTE_LAN in ${REMOTE_LANS}; do
		iptables -t nat -D VPNPLUS_SNAT_POSTROUTING -d ${REMOTE_LAN} -j MARK --set-xmark 0x10000000/0x10000000
		iptables -t filter -D VPNPLUS_S2S_ALLOW_FORWARD -s ${REMOTE_LAN} -j SYNO_FORWARD_ACCEPT
	done

}

AddL2TP()
{
	if [ ! -e "${L2TP_IPSEC_SECRETS}" ]; then
		$BIN_VPNTOOL -f -r
	fi

	# copy ipsec.conf and ipsec.secrets to /etc/ipsec.d
	/bin/cp -f ${L2TP_IPSEC_CONF} /etc/ipsec.d/ipsec_l2tp.conf
	/bin/chmod 400 ${L2TP_IPSEC_SECRETS}
	/bin/cp -f ${L2TP_IPSEC_SECRETS} /etc/ipsec.d/ipsec_l2tp.secrets
	/bin/rm -f ${L2TP_IPSEC_SECRETS}

	for f in /proc/sys/net/ipv4/conf/*;
	do
		echo 0 > $f/send_redirects;
		echo 0 > $f/accept_redirects;
	done

	local ipsecStatus=`/sbin/status ipsec`

	if [ "${ipsecStatus}" == "ipsec stop/waiting" ]; then
		start
	else
		${BIN_IPSEC} addconn --checkconfig
		${BIN_IPSEC} auto --add L2TP-PSK-NAT
		${BIN_IPSEC} auto --add L2TP-PSK-noNAT
		${BIN_IPSEC} whack --unlisten
		${BIN_IPSEC} whack --listen
	fi
}

DelL2TP()
{
	S2S_Enable=`${SZF_GET_SECTION_KEY_VALUE} ${SZF_VPNPLUS_CONF} ipsec enable`

	# remove ipsec.conf and ipsec.secrets in /etc/ipsec.d
	/bin/rm -f /etc/ipsec.d/ipsec_l2tp.conf
	/bin/rm -f /etc/ipsec.d/ipsec_l2tp.secrets

	if [ "yes" != ${S2S_Enable} ]; then
		stop
	else
		# ipsec will be closed when shutdown and reboot, so need to check to avoid block.
		if [ "`/sbin/status ipsec`" != "ipsec stop/waiting" ]; then
			${BIN_IPSEC} auto --delete L2TP-PSK-NAT
			${BIN_IPSEC} auto --delete L2TP-PSK-noNAT
			${BIN_IPSEC} addconn --checkconfig
			${BIN_IPSEC} auto --rereadsecrets
		fi
	fi
}

RemoveSecrets()
{
	for file in /etc/ipsec.d/*.secrets;
	do
		if [ -e "${file}" ]; then
			/bin/rm -f "${file}"
		fi
	done
}

case "$1" in
	start)
		start
	;;
	stop)
		stop
	;;
	restart)
		stop
		start
	;;
	add_tunnel)
		addtunnel "$2"
	;;
	del_tunnel)
		deltunnel "$2"
	;;
	add_l2tp)
		AddL2TP
	;;
	del_l2tp)
		DelL2TP
	;;
	*)
	echo "Usage: $0 {start|stop|restart|add_tunnel|del_tunnel|add_l2tp|del_l2tp}"
	exit 1
esac
exit 0
