#!/bin/sh
#
# Control script for openvpn server
#

PKG_USERCONF_DIR="/usr/syno/etc/packages/VPNPlusServer"
PKG_TARGET="/var/packages/VPNPlusServer/target"
CONF_DIR="${PKG_USERCONF_DIR}/openvpn"
OPENVPN_CONF="openvpn.conf"
IPTABLES_MODULE_LIST="/usr/syno/etc.defaults/iptables_modules_list"
SERVICE="openvpn"
BIN_IPTABLESTOOL="/usr/syno/bin/iptablestool"
BIN_SYNOMODULETOOL="/usr/syno/bin/synomoduletool"
LOCAL_PORT=`grep "^port " ${CONF_DIR}/${OPENVPN_CONF} | cut -d' ' -f 2`
LOCAL_PROTO=`grep "^proto " ${CONF_DIR}/${OPENVPN_CONF} | cut -d' ' -f 2`
SZF_NETTOOL="/usr/syno/sbin/synonettool"
LOCALIP=`grep "^server " ${CONF_DIR}/${OPENVPN_CONF} | cut -d' ' -f 2`
BIN_PORT443="${PKG_TARGET}/tool/synovpnplustool -w"
WEBSERVICE_NAME="httpd"
STR_WEB_BIND_PORT443="HTTP server listens on port 443"

SYNOVPNPLUS_CONF="${PKG_USERCONF_DIR}/synovpnplus.conf"
SECTION_OPENVPN="openvpn"
KEY_ENABLE="enable"
GET_KV="/bin/get_key_value"
OPENSSL="/usr/bin/openssl"

reverse_modules() {
	local modules=$1
	local mod
	local ret=""

	for mod in $modules; do
	    ret="$mod $ret"
	done

	echo $ret
}

Ovpn_Mod=""
if [ -f "${IPTABLES_MODULE_LIST}" ]; then
	. ${IPTABLES_MODULE_LIST}

	for mod in $OPENVPN_MODULES; do
		if [ -e "/lib/modules/$mod" ]; then
			Ovpn_Mod="${Ovpn_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_CORE; do
		if [ -e "/lib/modules/$mod" ]; then
			Ovpn_Mod="${Ovpn_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_COMMON; do
		if [ -e "/lib/modules/$mod" ]; then
			Ovpn_Mod="${Ovpn_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_NAT; do
		if [ -e "/lib/modules/$mod" ]; then
			Ovpn_Mod="${Ovpn_Mod} ${mod}"
		fi
	done
	for mod in $IPV6_MODULES; do
		if [ -e "/lib/modules/$mod" ]; then
			Ovpn_Mod="${Ovpn_Mod} ${mod}"
		fi
	done
fi

modify_push_route () {
	local TOPOLOGY=$1
	local ServerInfo=$(/bin/grep ^server "${CONF_DIR}/${OPENVPN_CONF}" | /usr/bin/awk '{ print $2 }')
	# clear routes except server route.
	/bin/grep route "${CONF_DIR}/${OPENVPN_CONF}" | /bin/grep -v "${ServerInfo}" | while read -r line
	do
		/bin/sed -i "/$line/d" "${CONF_DIR}/${OPENVPN_CONF}"
	done

	local pushRoute=$(get_section_key_value "${SYNOVPNPLUS_CONF}" openvpn push_route)

	if [ "yes" != "${pushRoute}" ]; then
		return
	fi

	if [ "bridge" = "${TOPOLOGY}" ]; then
		vpnifname=$(${SZF_NETTOOL} --wan --ifname 0)
		CurIP=$(/sbin/ip addr show "${vpnifname}" | /bin/grep "inet"  | /bin/grep -v "${vpnifname}:" | /bin/grep -v "inet6" | /usr/bin/awk '{ print $2 }')

		NewNet=$(/bin/ipcalc -n "${CurIP}" | /usr/bin/cut -d= -f 2)
		NewMask=$(/bin/ipcalc -m "${CurIP}" | /usr/bin/cut -d= -f 2)
		isExisted=$(/bin/grep -c "push \"route ${NewNet} ${NewMask}\"" "${CONF_DIR}/${OPENVPN_CONF}")
		if [ "0" = "$isExisted" ]; then
			echo "push \"route ${NewNet} ${NewMask}\"" >> "${CONF_DIR}/${OPENVPN_CONF}"
		fi
	else
		for vpnifname in $(${SZF_NETTOOL} --lan --list-ifnames); do
			if [ "gbr0" = "${vpnifname}" ]; then
				continue
			fi

			CurIP=$(/sbin/ip addr show "${vpnifname}" | /bin/grep "inet"  | /bin/grep -v "${vpnifname}:" | /bin/grep -v "inet6" | /usr/bin/awk '{ print $2 }')
			NewNet=$(/bin/ipcalc -n "${CurIP}" | /usr/bin/cut -d= -f 2)
			NewMask=$(/bin/ipcalc -m "${CurIP}" | /usr/bin/cut -d= -f 2)
			isExisted=$(/bin/grep -c "push \"route ${NewNet} ${NewMask}\"" "${CONF_DIR}/${OPENVPN_CONF}")
			if [ "0" = "$isExisted" ]; then
				echo "push \"route ${NewNet} ${NewMask}\"" >> "${CONF_DIR}/${OPENVPN_CONF}"
			fi
		done
	fi
}



start() {
	local topology=$(${SZF_NETTOOL} --topology --get)
	local enable=`get_section_key_value ${SYNOVPNPLUS_CONF} ${SECTION_OPENVPN} ${KEY_ENABLE}`
	if [ "no" = "${enable}" ]; then
		return
	fi

	local openvpnStatus=`/sbin/status openvpnServer`
	if [ "${openvpnStatus}" != "openvpnServer stop/waiting" ]; then
		echo "Already running"
		return
	fi

	echo -n "Starting OpenVPN server ..."

	if [ "${LOCAL_PORT}" = "443" -a "${LOCAL_PROTO}" = "tcp6-server" ]; then

		time=1
		status=`${BIN_PORT443} --get-443`

		if [ "${status}" = "${STR_WEB_BIND_PORT443}" ]; then
			${BIN_PORT443} --set-443-off
		fi

		while true; do
			/bin/sleep 1;

			if [ "$time" -ge "15" ]; then
				exit -1;
			fi

			status=`netstat -anp | grep ":443" | grep ${WEBSERVICE_NAME} | grep tcp | wc -l`
			time=$(($time + 1))

			if [ "$status" = "0" ]; then
				break;
			fi

			echo "wait for stop web server";

		done
	fi

	echo 1 > /proc/sys/net/ipv4/ip_forward

	# add SNAT
	if [ "bridge" = "${topology}" ]; then
		/sbin/iptables -t nat -A POSTROUTING -s ${LOCALIP}/24 -j MASQUERADE
	fi

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --insmod $SERVICE ${Ovpn_Mod}
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		$BIN_IPTABLESTOOL --insmod $SERVICE ${Ovpn_Mod}
	fi

	modify_push_route "${topology}"

        # copy certificate : ca.crt, ca_bundle.crt, server.crt, and server.key of ssl
	/bin/rm -f ${PKG_TARGET}/etc/openvpn/keys/ca_bundle.crt

	/bin/cp -f ${PKG_USERCONF_DIR}/openvpn/keys/ca.crt ${PKG_TARGET}/etc/openvpn/keys/
	/bin/cp -f ${PKG_USERCONF_DIR}/openvpn/keys/ca_bundle.crt ${PKG_TARGET}/etc/openvpn/keys/
	/bin/cp -f ${PKG_USERCONF_DIR}/openvpn/keys/server.crt ${PKG_TARGET}/etc/openvpn/keys/
	${OPENSSL} x509 -noout -in ${PKG_USERCONF_DIR}/openvpn/keys/server.crt  -subject | awk -F= '{print $NF}'| tr -d ' ' > ${PKG_TARGET}/etc/openvpn/keys/common_name
	/bin/cp -f ${PKG_USERCONF_DIR}/openvpn/keys/server.key ${PKG_TARGET}/etc/openvpn/keys/
	/bin/cp -f ${PKG_USERCONF_DIR}/openvpn/keys/ta.key ${PKG_TARGET}/etc/openvpn/keys/

	/sbin/start openvpnServer
}

stop() {
	local topology=$(${SZF_NETTOOL} --topology --get)
	local modules=$(reverse_modules "${Ovpn_Mod}")
	local openvpnStatus=$(/sbin/status openvpnServer)

	if [ "${openvpnStatus}" != "openvpnServer stop/waiting" ]; then
		echo -n "Shutting down OpenVPN server ..."
		/sbin/stop openvpnServer

		time=1
		while true; do
			/bin/sleep 1
			if [ $time -ge 10 ]; then
				break
			fi

			time=$(($time + 1))
			openvpnStatus=`/sbin/status openvpnServer`
			if [ "${openvpnStatus}" = "openvpnServer stop/waiting" ]; then
				break
			else
				echo "Wait for stoping openvpn server ..."
			fi
		done
	fi

	# del SNAT
	if [ "bridge" = "${topology}" ]; then
		/sbin/iptables -t nat -D POSTROUTING -s ${LOCALIP}/24 -j MASQUERADE
	fi

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --rmmod $SERVICE $modules
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		$BIN_IPTABLESTOOL --rmmod $SERVICE $modules
	fi
}
case "$1" in
	start)
		start
	;;
	stop)
		stop
	;;
	restart)
		stop
		start
	;;
	disconnect)
	        /usr/bin/curl "telnet://localhost:1195" < $2 > /dev/null 2>&1
	;;
	status)
		/usr/bin/curl "telnet://localhost:1195" < "/tmp/ovpn_cmd_status_2" > "/tmp/ovpn_status_2_result" 2>&1
	;;
	add_snat)
		if [ -e "${PID_FILE}" ]; then
			/sbin/iptables -t nat -A POSTROUTING -s ${LOCALIP}/24 -j MASQUERADE
		fi
	;;
	del_snat)
		if [ -e "${PID_FILE}" ]; then
			/sbin/iptables -t nat -D POSTROUTING -s ${LOCALIP}/24 -j MASQUERADE
		fi
	;;
	update_push_route)
		run=$(/usr/syno/bin/get_section_key_value ${SYNOVPNPLUS_CONF} openvpn enable)
		if [ "yes" != "${run}" ]; then
			return
		fi

		pushRoute=$(/usr/syno/bin/get_section_key_value "${SYNOVPNPLUS_CONF}" openvpn push_route)
		if [ "yes" != "${pushRoute}" ]; then
			return
		fi

		stop
		start
	;;
	*)
		echo "Usage: $0 {start|stop|restart|add_snat|del_snat|update_push_route}"
		exit 1
esac
exit 0
