#!/bin/sh
#
# Start/Stop the FreeRADIUS daemon
#

#
# Get the process ID of the FreeRADIUS daemon.
#
# @return
#	0: the PID is obtained and printed successfully
#	1: the PID file is not found (the daemon is not running)
#
get_daemon_pid ()
{
	local pid_file="/var/packages/VPNPlusServer/target/var/run/radiusd/radiusd.pid"

	if [ -f "${pid_file}" ]; then
		cat "${pid_file}"
		return 0
	else
		return 1
	fi
}

#
# Check whether the FreeRADIUS daemon is running or not.
#
# @return
#	0: the daemon is running
#	1: the daemon is not running
#
is_daemon_alive ()
{
	local daemon_pid=0

	daemon_pid=$(get_daemon_pid)
	if [ $? -eq 0 ]; then
		if [ -d "/proc/${daemon_pid}" ]; then
			return 0
		fi
	fi

	return 1
}

#
# Start the FreeRADIUS daemon.
#
# @return
#	0: success
#	1: failure
#
start ()
{
	local daemon_file="/var/packages/VPNPlusServer/target/sbin/vpnplusauthd"
	local daemon_pid=0

	if is_daemon_alive; then
		# return 0 if the daemon is already running
		return 0;
	fi

	# -t: please refer to bug [VPN Server] #215
	${daemon_file} -t

	if ! is_daemon_alive; then
		# return 1 if the daemon is not running as expected
		return 1;
	fi

	return 0;
}

#
# Stop the FreeRADIUS daemon.
#
# @return
#	0: success
#	1: failure
#
stop ()
{
	if ! is_daemon_alive; then
		# return 0 if the daemon is already dead
		return 0;
	fi

	daemon_pid=$(get_daemon_pid)
	if [ $? -eq 0 ]; then
		kill -TERM "${daemon_pid}"
	fi

	time=1
	while true; do
		/bin/sleep 1
		if [ $time -ge 10 ]; then
			break
		fi
		time=$(($time + 1))
		if [ -d /proc/${daemon_pid} ]; then
			echo "Wait for stoping FreeRADIUS ..."
		else
			break
		fi
	done

	if is_daemon_alive; then
		# return 1 if the daemon is not dead as expected
		return 1;
	fi

	return 0;
}

case "$1" in
	start)
		start
	;;
	stop)
		stop
	;;
	restart)
		stop
		start
	;;
	*)
		echo "Usage: $0 {start|stop|restart}"
		exit 1
	;;
esac
exit $?
