#!/bin/sh

#################################################################
# This hook is invoked when ip change
# For renew push "route {private network}"
# Usable environment variable: IFNAME,ORIGIN_ADDRESS,NEW_ADDRESS
#################################################################

PACKAGE_NAME="VPNPlusServer"
PACKAGE_TARGET_DIR="/var/packages/${PACKAGE_NAME}/target"
PACKAGE_USR_CONF_DIR="/var/packages/${PACKAGE_NAME}/etc"
VPNPLUS_CONFIG="${PACKAGE_USR_CONF_DIR}/synovpnplus.conf"
OPENVPN_CONFIG="${PACKAGE_USR_CONF_DIR}/openvpn/openvpn.conf"
SZF_NETTOOL="/usr/syno/sbin/synonettool"

modify_push_route () {
	local ServerInfo=$(/bin/grep ^server "${OPENVPN_CONFIG}" | /usr/bin/awk '{ print $2 }')

	# clear routes except server route.
	/bin/grep route "${OPENVPN_CONFIG}" | /bin/grep -v "${ServerInfo}" | while read -r line
	do
		/bin/sed -i "/$line/d" "${OPENVPN_CONFIG}"
	done

	local pushRoute=$(get_section_key_value "${VPNPLUS_CONFIG}" openvpn push_route)
	if [ "yes" != "${pushRoute}" ]; then
		return
	fi

	if [ "bridge" = "${TOPOLOGY}" ]; then
		vpnifname=$(${SZF_NETTOOL} --wan --ifname 0)
		CurIP=$(/sbin/ip addr show "${vpnifname}" | /bin/grep "inet"  | /bin/grep -v "${vpnifname}:" | /bin/grep -v "inet6" | /usr/bin/awk '{ print $2 }')

		NewNet=$(/bin/ipcalc -n "${CurIP}" | /usr/bin/cut -d= -f 2)
		NewMask=$(/bin/ipcalc -m "${CurIP}" | /usr/bin/cut -d= -f 2)
		isExisted=$(/bin/grep -c "push \"route ${NewNet} ${NewMask}\"" "${OPENVPN_CONFIG}")
		if [ "0" = "$isExisted" ]; then
			echo "push \"route ${NewNet} ${NewMask}\"" >> "${OPENVPN_CONFIG}"
		fi
	else
		for vpnifname in $(${SZF_NETTOOL} --lan --list-ifnames); do
			if [ "gbr0" = "${vpnifname}" ]; then
				continue
			fi

			CurIP=$(/sbin/ip addr show "${vpnifname}" | /bin/grep "inet"  | /bin/grep -v "${vpnifname}:" | /bin/grep -v "inet6" | /usr/bin/awk '{ print $2 }')
			NewNet=$(/bin/ipcalc -n "${CurIP}" | /usr/bin/cut -d= -f 2)
			NewMask=$(/bin/ipcalc -m "${CurIP}" | /usr/bin/cut -d= -f 2)
			isExisted=$(/bin/grep -c "push \"route ${NewNet} ${NewMask}\"" "${OPENVPN_CONFIG}")
			if [ "0" = "$isExisted" ]; then
				echo "push \"route ${NewNet} ${NewMask}\"" >> "${OPENVPN_CONFIG}"
			fi
		done
	fi

	${PACKAGE_TARGET_DIR}/scripts/openvpn.sh restart
}

case $1 in
	--sdk-mod-ver)
		echo "1.0"
		;;
	--name)
		;;
	--pkg-ver)
		;;
	--vendor)
		;;
	--pre)
		;;
	--post)
		if [ "0.0.0.0" = "${NEW_ADDRESS}" ]; then
			return
		fi

		local run=$(/usr/syno/bin/get_section_key_value ${VPNPLUS_CONFIG} openvpn enable)
		local is_lan=$(${SZF_NETTOOL} --lan --list-ifnames | grep -c "${IFNAME}")
		if [ "yes" = "${run}" ] && [ "1" = "${is_lan}" ] && [ "gbr0" != "${IFNAME}" ]; then
			modify_push_route
		fi
		;;
	*)
		echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
		;;
esac
