#!/bin/sh
#
# Control script for sslvpn server
#

PKG_USERCONF_DIR="/usr/syno/etc/packages/VPNPlusServer"
PKG_TARGET="/var/packages/VPNPlusServer/target"
CONF_DIR="${PKG_USERCONF_DIR}/sslvpn"
SSLVPN_CONF="vpnserver.conf"
IPTABLES_MODULE_LIST="/usr/syno/etc.defaults/iptables_modules_list"
SERVICE="sslvpn"
PID_FILE="${PKG_TARGET}/var/run/vpnserver.pid"
BINARY_NAME="vpnserver"
SZF_VPNTOOL="${PKG_TARGET}/tool/synovpnplustool"
BIN_IPTABLESTOOL="/usr/syno/bin/iptablestool"
BIN_SYNOMODULETOOL="/usr/syno/bin/synomoduletool"
TAP_NAME_SSL="tap_vpn"
TAP_NAME_SSTP="tap_sstp"

SYNOVPNPLUS_CONF="${PKG_USERCONF_DIR}/synovpnplus.conf"
SECTION_SSLVPN="sslvpn"
SECTION_SSTP="sstp"
KEY_ENABLE="enable"
KEY_IPPOOL="ippool"
multiplexer_mark="0x20000"
multiplexer_mask="0x20000"
multiplexer_table="100"
GET_KV="synogetkeyvalue"

reverse_modules() {
	local modules=$1
	local mod
	local ret=""

	for mod in $modules; do
	    ret="$mod $ret"
	done

	echo $ret
}

#same as openvpn
Ovpn_Mod=""
if [ -f "${IPTABLES_MODULE_LIST}" ]; then
	. ${IPTABLES_MODULE_LIST}

	for mod in $OPENVPN_MODULES; do
		if [ -e "/lib/modules/$mod" ]; then
			Ovpn_Mod="${Ovpn_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_CORE; do
		if [ -e "/lib/modules/$mod" ]; then
			Ovpn_Mod="${Ovpn_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_COMMON; do
		if [ -e "/lib/modules/$mod" ]; then
			Ovpn_Mod="${Ovpn_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_NAT; do
		if [ -e "/lib/modules/$mod" ]; then
			Ovpn_Mod="${Ovpn_Mod} ${mod}"
		fi
	done
	for mod in $IPV6_MODULES; do
		if [ -e "/lib/modules/$mod" ]; then
			Ovpn_Mod="${Ovpn_Mod} ${mod}"
		fi
	done
fi

install_iptables() {
	port=`cat /var/packages/VPNPlusServer/etc/sslvpn/vpnserver.conf | python -c 'import sys, json; print json.load(sys.stdin)["webportal"]["port"]'`
	iptables -t mangle -A MULTIPLEXER_OUTPUT -p tcp ! -o lo --sport $port -j MARK --set-mark $multiplexer_mask/$multiplexer_mark
	iptables -t mangle -I MULTIPLEXER_POSTROUTING -m mark --mark $multiplexer_mask/$multiplexer_mark -j MARK --set-mark 0/$multiplexer_mark
	ip route add local 0.0.0.0/0 dev lo table $multiplexer_table
	ip rule add fwmark $multiplexer_mask/$multiplexer_mark lookup $multiplexer_table

	ip6tables -t mangle -A MULTIPLEXER_OUTPUT -p tcp ! -o lo --sport $port -j MARK --set-mark $multiplexer_mask/$multiplexer_mark
	ip6tables -t mangle -I MULTIPLEXER_POSTROUTING -m mark --mark $multiplexer_mask/$multiplexer_mark -j MARK --set-mark 0/$multiplexer_mark
	ip -6 route add local ::/0 dev lo table $multiplexer_table
	ip -6 rule add fwmark $multiplexer_mask/$multiplexer_mark lookup $multiplexer_table priority 1
}

uninstall_iptables() {
	iptables -t mangle -F MULTIPLEXER_OUTPUT
	iptables -t mangle -F MULTIPLEXER_POSTROUTING
	ip rule del fwmark $multiplexer_mask/$multiplexer_mark lookup $multiplexer_table
	ip route del table $multiplexer_table

	ip6tables -t mangle -F MULTIPLEXER_OUTPUT
	ip6tables -t mangle -F MULTIPLEXER_POSTROUTING
	ip -6 rule del fwmark $multiplexer_mask/$multiplexer_mark lookup $multiplexer_table
	ip -6 route del table $multiplexer_table
}

reset_iptables() {
	uninstall_iptables
	install_iptables
}

check_process() {
	if [ -e "${PID_FILE}" ]; then
		local pid=`cat ${PID_FILE}`
		if [ -e "/proc/${pid}" ]; then
			return 1
		else
			rm ${PID_FILE}
		fi
	fi
	return 0
}

wait_interface() {
	local ifname=$1

	i=0
	while [ "$i" -lt 15 ]
	do
		i=$(($i+1))
		local ret=`/sbin/ifconfig ${ifname} 2>/dev/null`
		if [ -n "$ret" ]; then
			break
		fi
		sleep 1
	done
}

reload_conf() {
	ret=`netstat -anp | grep ${BINARY_NAME} | grep LISTEN | awk '{print $4}'`
	for info in $ret
	do
		curl -k https://${info}/@config/reload.cgi 1>/dev/null 2>/dev/null
		break;
	done
}

reload_route_rules() {
	ret=`netstat -anp | grep ${BINARY_NAME} | grep LISTEN | awk '{print $4}'`
	for info in $ret
	do
		curl -k https://${info}/@config/reload_route.cgi 1>/dev/null 2>/dev/null
		break;
	done
}

check_bridge() {
	local enable_ssl=`get_section_key_value ${SYNOVPNPLUS_CONF} ${SECTION_SSLVPN} ${KEY_ENABLE}`
	local enable_sstp=`get_section_key_value ${SYNOVPNPLUS_CONF} ${SECTION_SSTP} ${KEY_ENABLE}`
	local ippool_ssl=`get_section_key_value ${SYNOVPNPLUS_CONF} ${SECTION_SSLVPN} ${KEY_IPPOOL}`
	local ippool_sstp=`get_section_key_value ${SYNOVPNPLUS_CONF} ${SECTION_SSTP} ${KEY_IPPOOL}`
	local ifcfg_ssl="/etc/sysconfig/network-scripts/ifcfg-${TAP_NAME_SSL}"
	local ifcfg_sstp="/etc/sysconfig/network-scripts/ifcfg-${TAP_NAME_SSTP}"

	if [ "yes" == "${enable_ssl}" ]; then
		wait_interface $TAP_NAME_SSL
		local ret=`brctl show | grep $TAP_NAME_SSL`
		if [ -z "${ret}" ]; then
			brctl addif $ippool_ssl $TAP_NAME_SSL
			${SZF_VPNTOOL} -u -s ${ifcfg_ssl} BRIDGE ${ippool_ssl}
			${SZF_VPNTOOL} -u -s ${ifcfg_ssl} DEVICE ${TAP_NAME_SSL}
			${SZF_VPNTOOL} -u -s ${ifcfg_ssl} BOOTPROTO static
		fi
	fi
	if [ "yes" == "${enable_sstp}" ]; then
		wait_interface $TAP_NAME_SSTP
		local ret=`brctl show | grep $TAP_NAME_SSTP`
		if [ -z "${ret}" ]; then
			brctl addif $ippool_sstp $TAP_NAME_SSTP
			${SZF_VPNTOOL} -u -s ${ifcfg_sstp} BRIDGE ${ippool_sstp}
			${SZF_VPNTOOL} -u -s ${ifcfg_sstp} DEVICE ${TAP_NAME_SSTP}
			${SZF_VPNTOOL} -u -s ${ifcfg_sstp} BOOTPROTO static
		fi
	fi

	${SZF_VPNTOOL} -v -u
}

start_daemon() {
	/sbin/start vpnServer
	check_bridge
	install_iptables
}

stop_daemon() {
	/sbin/stop vpnServer
	uninstall_iptables

	${SZF_VPNTOOL} -v -u
}

reload_daemon() {
	reload_conf
	check_bridge
	reset_iptables
}

start() {
	local enable_ssl=`get_section_key_value ${SYNOVPNPLUS_CONF} ${SECTION_SSLVPN} ${KEY_ENABLE}`
	local enable_sstp=`get_section_key_value ${SYNOVPNPLUS_CONF} ${SECTION_SSTP} ${KEY_ENABLE}`

	check_process
	local status=$?

	if [ "yes" == "${enable_ssl}" -o "yes" == "${enable_sstp}" ]; then
		if [ "$status" -eq 0 ]; then
			echo -n "Starting SSL VPN server ..."
			echo 1 > /proc/sys/net/ipv4/ip_forward

			if [ -x ${BIN_SYNOMODULETOOL} ]; then
				$BIN_SYNOMODULETOOL --insmod $SERVICE ${Ovpn_Mod}
			elif [ -x ${BIN_IPTABLESTOOL} ]; then
				$BIN_IPTABLESTOOL --insmod $SERVICE ${Ovpn_Mod}
			fi
			start_daemon
		else
			reload_daemon
		fi
	fi
}

stop() {
	local modules=`reverse_modules "${Ovpn_Mod}"`
	local enable_ssl=`get_section_key_value ${SYNOVPNPLUS_CONF} ${SECTION_SSLVPN} ${KEY_ENABLE}`
	local enable_sstp=`get_section_key_value ${SYNOVPNPLUS_CONF} ${SECTION_SSTP} ${KEY_ENABLE}`

	check_process
	local status=$?

	if [ "$status" -eq 0 ]; then
		return;
	fi

	if [ "yes" == "${enable_ssl}" -o "yes" == "${enable_sstp}" ]; then
		reload_daemon
		return
	fi

	stop_daemon

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --rmmod $SERVICE $modules
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		$BIN_IPTABLESTOOL --rmmod $SERVICE $modules
	fi
}

reload() {
	check_process
	local status=$?

	if [ "$status" -eq 0 ]; then
		exit 0;
	fi
	reload_daemon
}

reload_route() {
	check_process
	local status=$?

	if [ "$status" -eq 0 ]; then
		exit 0;
	fi
	reload_route_rules
}

case "$1" in
	start)
		start
	;;
	stop)
		stop
	;;
	reload)
		reload
	;;
	reload_route)
		reload_route
	;;
	restart)
		stop
		start
	;;
	force-stop)
		stop_daemon
	;;
	force-restart)
		# Force restart daemon without checkout enable config
		check_process
		local status=$?
		if [ "$status" -eq 1 ]; then
			stop_daemon
			start_daemon
		fi
	;;
	*)
		echo "Usage: $0 {start|stop|restart|force-stop|force-restart}"
		exit 1
esac
exit 0
