#!/bin/sh

PKG_DIR="/var/packages/VPNPlusServer"
PKG_USR_CONF_DIR="${PKG_DIR}/etc"
DB_FILE="${PKG_USR_CONF_DIR}/db/synovpnplus.db"
VPNPLUS_CONF="${PKG_USR_CONF_DIR}/synovpnplus.conf"

PYTHON="/usr/bin/python"
SZF_LOGGER="/usr/bin/logger"
SQLITE3="/usr/syno/bin/sqlite3"
GET_SEC_KV="/usr/syno/bin/get_section_key_value"

del_half_data()
{
	${SQLITE3} ${DB_FILE} "DELETE FROM traffic_data WHERE td_id IN (SELECT td_id FROM traffic_data ORDER BY time ASC LIMIT (SELECT COUNT(*) FROM traffic_data)/2)"
	${SZF_LOGGER} -p info -t synovpnplus "table 'traffic_data' rotated."

	${SQLITE3} ${DB_FILE} "DELETE FROM traffic_protocol_data WHERE tpd_id IN (SELECT tpd_id FROM traffic_protocol_data ORDER BY time ASC LIMIT (SELECT COUNT(*) FROM traffic_protocol_data)/2)"
	${SZF_LOGGER} -p info -t synovpnplus "table 'traffic_protocol_data' rotated."

	${SQLITE3} ${DB_FILE} "DELETE FROM user_log WHERE ul_id IN (SELECT ul_id FROM user_log ORDER BY time ASC LIMIT (SELECT COUNT(*) FROM user_log)/2)"
	${SZF_LOGGER} -p info -t synovpnplus "table 'user_log' rotated."

	${SQLITE3} ${DB_FILE} "DELETE FROM user_history WHERE uh_id IN (SELECT uh_id FROM user_history ORDER BY time ASC LIMIT (SELECT COUNT(*) FROM user_history)/2)"
	${SZF_LOGGER} -p info -t synovpnplus "delete old data of 'user_history'."
}

del_older_data()
{
	max_log_day=`${GET_SEC_KV} ${VPNPLUS_CONF} Traffic max_log_day`
	cur_time=`date +%s`
	old_time=$((${cur_time} - (86400 * ${max_log_day})))

	${SQLITE3} ${DB_FILE} "PRAGMA foreign_keys=on; DELETE FROM connectivity WHERE time_stop < ${old_time}"
	${SZF_LOGGER} -p info -t synovpnplus "delete old data of 'connectivity'."

	${SQLITE3} ${DB_FILE} "DELETE FROM traffic_data WHERE time < ${old_time}"
	${SZF_LOGGER} -p info -t synovpnplus "delete old data of 'traffic_date'."

	${SQLITE3} ${DB_FILE} "DELETE FROM traffic_protocol_data WHERE time < ${old_time}"
	${SZF_LOGGER} -p info -t synovpnplus "delete old data of 'traffic_protocol_data'."

	${SQLITE3} ${DB_FILE} "DELETE FROM traffic_s2s_data WHERE time < ${old_time}"
	${SZF_LOGGER} -p info -t synovpnplus "delete old data of 'traffic_s2s_data'."

	${SQLITE3} ${DB_FILE} "DELETE FROM user_log WHERE time < ${old_time}"
	${SZF_LOGGER} -p info -t synovpnplus "delete old data of 'user_log'."
}

del_older_report_data()
{
	max_log_day=150
	cur_time=`date +%s`
	old_time=$((${cur_time} - (86400 * ${max_log_day})))

	${SQLITE3} ${DB_FILE} "DELETE FROM traffic_data_report WHERE time < ${old_time}"
	${SZF_LOGGER} -p info -t synovpnplus "delete old data of 'traffic_date_report'."

	${SQLITE3} ${DB_FILE} "DELETE FROM traffic_protocol_data_report WHERE time < ${old_time}"
	${SZF_LOGGER} -p info -t synovpnplus "delete old data of 'traffic_protocol_data_report'."

	${SQLITE3} ${DB_FILE} "DELETE FROM traffic_s2s_data_report WHERE time < ${old_time}"
	${SZF_LOGGER} -p info -t synovpnplus "delete old data of 'traffic_s2s_data_report'."
}

main()
{
	del_older_data
	del_older_report_data

	${SQLITE3} ${DB_FILE} "VACUUM"

	max_log_size=`${GET_SEC_KV} ${VPNPLUS_CONF} Traffic max_log_size`
	max_size=`${PYTHON} -c "print int(${max_log_size} * 1024 * 1024)"`
	db_size=`du "${DB_FILE}" | awk '{print $1}'`

	if [ "${db_size}" -gt "${max_size}" ]; then
		del_half_data
		${SQLITE3} ${DB_FILE} "VACUUM"
	fi
}

main "$@"
