#!/bin/sh
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.
PKG_DIR="/var/packages/VPNPlusServer"
PKG_TARGET_DIR="${PKG_DIR}/target"
PKG_USR_CONF_DIR="${PKG_DIR}/etc"
UPDATER_LOG="${PKG_USR_CONF_DIR}/updater.log"
UPDATER_UTILS="${PKG_TARGET_DIR}/scripts/updater_utils.sh"
SH_PATCH_DIR="${PKG_TARGET_DIR}/scripts/patch"

oldVersion=$1
newVersion=$2

. ${UPDATER_UTILS}

if [ ! -e "${UPDATER_LOG}" ]; then
	touch ${UPDATER_LOG}
fi

echo  "==========Start==========" >> ${UPDATER_LOG}
echo  "Run updater scripts from ${oldVersion} to ${newVersion}" >> ${UPDATER_LOG}

isVersionLessEqual() # 0 is true, 1 is false
{
	BASE_VER=$1
	TARGET_VER=$2
	# if no base or target version, always reture falsel
	if [ -z "${BASE_VER}" ] || [ -z "${TARGET_VER}" ]; then
		return 1;
	fi

	# getting major, minor, build
	base_1=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\1/'`
	base_2=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\2/'`
	base_3=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\3/'`
	base_4=`echo ${BASE_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\4/'`
	target_1=`echo ${TARGET_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\1/'`
	target_2=`echo ${TARGET_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\2/'`
	target_3=`echo ${TARGET_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\3/'`
	target_4=`echo ${TARGET_VER} | sed 's/^\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\)[.-]\([0-9]*\).*/\4/'`

	# compare first
	if [ $target_1 -lt $base_1 ]; then
		return 1;
	elif [ $target_1 -gt $base_1 ]; then
		return 0;
	fi
	# compare second
	if [ $target_2 -lt $base_2 ]; then
		return 1;
	elif [ $target_2 -gt $base_2 ]; then
		return 0;
	fi
	# compare third
	if [ $target_3 -lt $base_3 ]; then
		return 1;
	elif [ $target_3 -gt $base_3 ]; then
		return 0;
	fi
	# compare fourth
	if [ $target_4 -lt $base_4 ]; then
		return 1;
	else
		return 0;
	fi
}

# put patch file into Array
numPatch=1
for i in `ls $SH_PATCH_DIR`
do
	eval "Array_$numPatch=$i"
	numPatch=$(($numPatch+1))
done
numPatch=$(($numPatch-1))

# sort patch file ascending in version
for i in $(seq 1 $numPatch)
do
	for k in $(seq $i $numPatch)
	do
		eval A1=\$Array_$i
		eval A2=\$Array_$k
		if isVersionLessEqual $A2 $A1; then
			eval "tmp=\$Array_$i"
			eval "Array_$i=\$Array_$k"
			eval "Array_$k=$tmp"
		fi
	done
done

# apply patch if ( oldVersion < patch <= newVersion )
for i in $(seq 1 $numPatch)
do
	eval patch=\$Array_$i
	test $patch == $oldVersion.sh && continue

	if isVersionLessEqual $oldVersion $patch; then
		if isVersionLessEqual $patch $newVersion; then
			echo  "run ${SH_PATCH_DIR}/${patch} " >> ${UPDATER_LOG}
			. ${SH_PATCH_DIR}/${patch}
			echo  "result $?" >> ${UPDATER_LOG}
		fi
	fi
done

echo  "finish!" >> ${UPDATER_LOG}

exit 0
