#!/bin/sh

#################################################################
# This hook is invoked when topology change
#################################################################

PACKAGE_LOCATION="/var/packages/VPNPlusServer"
SZF_IPPOOL_SH="${PACKAGE_LOCATION}/target/scripts/ippool.sh"
SZF_OPENVPN_SH="${PACKAGE_LOCATION}/target/scripts/openvpn.sh"
SZF_TOPOLOGY_PY="${PACKAGE_LOCATION}/target/scripts/vpnplus-topology-lan-change.py"
SZF_TOPOLOGY_LAN_PY="${PACKAGE_LOCATION}/target/scripts/vpnplus-topology-lan-utils.py"
SZF_VPNTOOL="${PACKAGE_LOCATION}/target/tool/synovpnplustool"
VPNPLUS_CONF="${PACKAGE_LOCATION}/etc/synovpnplus.conf"
SZF_NETTOOL="/usr/syno/sbin/synonettool"
PYTHON="/usr/bin/python2.7"

add_snat() {
	NEW_TOPOLOGY=$2
	if [ "${NEW_TOPOLOGY}" = "bridge" ]; then
		${PACKAGE_LOCATION}/target/scripts/ippool.sh add-all-snat "$1" "$NEW_TOPOLOGY"
		${PACKAGE_LOCATION}/target/scripts/openvpn.sh add_snat
	fi
}

del_snat() {
	OLD_TOPOLOGY=$2
	if [ "${OLD_TOPOLOGY}" = "bridge" ]; then
		${PACKAGE_LOCATION}/target/scripts/ippool.sh del-all-snat "$1" "$OLD_TOPOLOGY"
		${PACKAGE_LOCATION}/target/scripts/openvpn.sh del_snat
	fi
}

case $1 in
	--sdk-mod-ver)
	#Print SDK support version
		echo "1.0";
	;;
	--name)
	#Print package name
		echo "VPNPlus";
	;;
	--pkg-ver)
	#Print package version
		echo "1.0";
	;;
	--vendor)
	#Print package vendor
		echo "Synology";
	;;
	--pre)
		OLD_TOPOLOGY=$(jq -r '.old.context' "${ARGS_FILE_PATH}")
		NEW_TOPOLOGY=$(jq -r '.new.context' "${ARGS_FILE_PATH}")

		if [ "x${NEW_TOPOLOGY}" == "x${OLD_TOPOLOGY}" ]; then
			return
		fi

		lans=$(${PYTHON} ${SZF_TOPOLOGY_LAN_PY} "${ARGS_FILE_PATH}" "old")
		del_snat "${lans}" "${OLD_TOPOLOGY}"
		if [ "x${NEW_TOPOLOGY}" == "xbridge" ]; then
			for interface in ${lans}; do
				${SZF_IPPOOL_SH} stopTap "${interface}"
			done
		fi
	;;
	--post)
		OLD_TOPOLOGY=$(jq -r '.old.context' "${ARGS_FILE_PATH}")
		NEW_TOPOLOGY=$(jq -r '.new.context' "${ARGS_FILE_PATH}")

		${PYTHON} "${SZF_TOPOLOGY_PY}" "${ARGS_FILE_PATH}"
		if [ "x${NEW_TOPOLOGY}" == "x${OLD_TOPOLOGY}" ]; then
			return
		fi

		${SZF_VPNTOOL} -t -r
		lans=$(${PYTHON} ${SZF_TOPOLOGY_LAN_PY} "${ARGS_FILE_PATH}" "new")
		add_snat "${lans}" "${NEW_TOPOLOGY}"
		if [ "x${OLD_TOPOLOGY}" == "xbridge" ]; then
			for interface in $(${SZF_NETTOOL} --lan --list-ifnames); do
				${SZF_IPPOOL_SH} startTap "${interface}"
			done
		fi
		${SZF_VPNTOOL} -f -u # Auto update IF
		${SZF_OPENVPN_SH} restart
	;;
	*)
	echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
	;;
esac


