#!/usr/bin/env python2.7

'''
{
    'old': {
	    'context': 'router',
	    'networks': {
		    'lbr0': {
			    'owners': [ 'LAN' ],
		    },
		    'gbr0': {
			    'owners': [ 'LAN' ],
		    },
		    'voiptv0': {
			    'owners': [ 'VOIPTV' ],
		    },
		    'wan0': {
			    'ifname': 'eth0',
			    'owners': [ 'VOIPTV', 'WAN' ],
		    },
		    'wan1': {
			    'ifname': 'eth2',
			    'owners': [ 'WAN' ],
		    }
	    }
    },
    'new': {
	    'context': 'bridge',
	    'networks': {
		    'br0': {
			    'owners': [ 'WAN', 'LAN' ],
		    },
		    'br2': {
			    'owners': [ 'LAN' ],
		    },
		    'gbr0': {
			    'owners': [ 'LAN' ],
		    }
	    }
    }
}
'''

import json
import argparse
import sys
import subprocess

vpntool = '/var/packages/VPNPlusServer/target/tool/synovpnplustool'
ovpn = '/var/packages/VPNPlusServer/target/scripts/openvpn.sh'

def getLans(networks):
    lans = set()
    for ifname, value in networks.items():
        if 'LAN' in value['owners'] and 1 == len(value['owners']):
            lans.add(ifname)

    return lans

def addLan(ifname):
    subprocess.call([vpntool, '-f', '-e', ifname])

def delLan(ifname):
    subprocess.call([vpntool, '-f', '-j', ifname])

def updatePushRoute():
    subprocess.call([ovpn, 'update_push_route'])

def main():
    parser = argparse.ArgumentParser(description='vpn topology change')
    parser.add_argument('topology_event_file', help='topology event file')

    args = parser.parse_args()

    f = open(args.topology_event_file)
    data = json.load(f)
    f.close()

    oldLans = getLans(data['old']['networks'])
    newLans = getLans(data['new']['networks'])

    # old obj exists, but new obj doesn't exist
    # we need to remove the element in VPN
    for ifname in oldLans - newLans:
        delLan(ifname)

    # new obj exists, but old obj doesn't exist
    # we need to add the element in VPN
    for ifname in newLans - oldLans:
        addLan(ifname)

    if oldLans != newLans and data['old']['context'] == data['new']['context']:
        updatePushRoute()

if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        print (e)
        sys.exit(-1)
    except:
        print ('unknown error')
        sys.exit(-1)

    sys.exit(0)
