#!/bin/sh
#
# Control script for L2TP server
#

PKG_USERCONF_DIR="/usr/syno/etc/packages/VPNPlusServer"
PKG_TARGET="/var/packages/VPNPlusServer/target"
LOCALIP=`/bin/get_key_value ${PKG_USERCONF_DIR}/l2tp/xl2tpd.conf "local ip"`
IPTABLES_MODULE_LIST="/usr/syno/etc.defaults/iptables_modules_list"
SERVICE="l2tp"
BIN_IPTABLESTOOL="/usr/syno/bin/iptablestool"
BIN_SYNOMODULETOOL="/usr/syno/bin/synomoduletool"
MTU=`/bin/cat ${PKG_USERCONF_DIR}/l2tp/options.xl2tpd | grep mtu | sed 's/^.*mtu //g'`
MSS=$((MTU-44))
BIN_VPNTOOL="${PKG_TARGET}/tool/synovpnplustool"
GET_KV="/bin/get_key_value"
IPSEC_SCRIPT="${PKG_TARGET}/scripts/ipsec.sh"

reverse_modules() {
	local modules=$1
	local mod
	local ret=""

	for mod in $modules; do
	    ret="$mod $ret"
	done

	echo $ret
}

L2TP_IPsec_Mod=""
if [ -f "${IPTABLES_MODULE_LIST}" ]; then
	. ${IPTABLES_MODULE_LIST}

	for mod in $PPP_MODULES; do
		if [ -e "/lib/modules/$mod" ]; then
			L2TP_IPsec_Mod="${L2TP_IPsec_Mod} ${mod}"
		fi
	done
	for mod in $L2TP_MODULES; do
		if [ -e "/lib/modules/$mod" ]; then
			L2TP_IPsec_Mod="${L2TP_IPsec_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_CORE; do
		if [ -e "/lib/modules/$mod" ]; then
			L2TP_IPsec_Mod="${L2TP_IPsec_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_COMMON; do
		if [ -e "/lib/modules/$mod" ]; then
			L2TP_IPsec_Mod="${L2TP_IPsec_Mod} ${mod}"
		fi
	done
	for mod in $KERNEL_MODULES_NAT; do
		if [ -e "/lib/modules/$mod" ]; then
			L2TP_IPsec_Mod="${L2TP_IPsec_Mod} ${mod}"
		fi
	done
fi

start() {
	local TOPOLOGY=$(/usr/syno/sbin/synonettool --topology --get)
	l2tpStatus=`/sbin/status l2tpServer`

	if [ "${l2tpStatus}" != "l2tpServer stop/waiting" ]; then
		echo "Already running"
		return
	fi

	echo -n "Starting L2TP server ..."
	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --insmod $SERVICE ${L2TP_IPsec_Mod}
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		$BIN_IPTABLESTOOL --insmod $SERVICE ${L2TP_IPsec_Mod}
	fi

	if [ ! -d "/var/run/xl2tpd" ]; then
		/bin/mkdir -p /var/run/xl2tpd
	fi

	echo 1 > /proc/sys/net/ipv4/ip_forward

	# deal with iptable rules
	/sbin/iptables -t mangle -A PRIMITIVE_FORWARD -i ppp3+ -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --set-mss "${MSS}"
	/sbin/iptables -A VPN_SERVER_INPUT -p udp --dport 1701 -m policy --dir in --pol ipsec -j SYNO_INPUT_ACCEPT
	/sbin/iptables -A VPN_SERVER_INPUT -p udp --dport 1701 -j DROP
	if [ "bridge" != "${TOPOLOGY}" ]; then
		/sbin/iptables -t mangle -A PRIMITIVE_FORWARD -i ppp3+ -m mark --mark 0x10000/0x10000 -j MARK --set-xmark 0x0/0x10000
	fi

	/sbin/start l2tpServer

	${IPSEC_SCRIPT} add_l2tp
}

stop() {
	local TOPOLOGY=$(/usr/syno/sbin/synonettool --topology --get)
	local modules=`reverse_modules "${L2TP_IPsec_Mod}"`
	local l2tpStatus=`/sbin/status l2tpServer`

	if [ "${l2tpStatus}" == "l2tpServer stop/waiting" ]; then
		echo "Already stoped"
		return
	fi

	#disconnect clients
	if [ -n "${LOCALIP}" ]; then
		/bin/kill `ps -I ww | /bin/grep pppd | /bin/grep l2tp | /bin/grep "${LOCALIP}" | awk '{print $1;}'` 2>/dev/null
	fi

	# deal with iptable rules
	/sbin/iptables -t mangle -D PRIMITIVE_FORWARD -i ppp3+ -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --set-mss "${MSS}"
	/sbin/iptables -D VPN_SERVER_INPUT -p udp --dport 1701 -m policy --dir in --pol ipsec -j SYNO_INPUT_ACCEPT
	/sbin/iptables -D VPN_SERVER_INPUT -p udp --dport 1701 -j DROP
	if [ "bridge" != "${TOPOLOGY}" ]; then
		/sbin/iptables -t mangle -D PRIMITIVE_FORWARD -i ppp3+ -m mark --mark 0x10000/0x10000 -j MARK --set-xmark 0x0/0x10000
	fi

	/sbin/stop l2tpServer

	echo -n "Shutting down L2TP server ..."
	local time=1

	while true; do
		/bin/sleep 1
		if [ $time -ge 10 ]; then
			break
		fi

		time=$(($time + 1))
		l2tpStatus=`/sbin/status l2tpServer`

		if [ "${l2tpStatus}" = "l2tpServer stop/waiting" ]; then
			break
		else
			echo "Wait for stoping l2tp server ..."
		fi
	done

	${IPSEC_SCRIPT} del_l2tp

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --rmmod $SERVICE $modules
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		$BIN_IPTABLESTOOL --rmmod $SERVICE $modules
	fi
}

(
flock -x 200
case "$1" in
	start)
		start
	;;
	stop)
		stop
	;;
	restart)
		stop
		start
	;;
	*)
	echo "Usage: $0 {start|stop|restart}"
	exit 1
esac
) 200>/var/lock/xl2tp.lock
exit 0
